__author__    = 'Viktor Kerkez <alefnula@teacupapps.com>'
__contact__   = 'alefnula@teacupapps.com'
__date__      = '03 January 2010'
__copyright__ = 'Copyright (c) 2010 Viktor Kerkez'

__all__ = ['MSG_INFO', 'MSG_WARNING', 'MSG_ERROR',
           'QUESTION_YES_NO',
           'DIALOG_DIRECTORY_OPEN', 'DIALOG_FILE_OPEN', 'DIALOG_FILE_SAVE',
           'DIALOG_SELECT_FONT', 'DIALOG_SELECT_COLOR'
]

# PyQt4 imports
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt


def MSG_INFO(text, title='Information', parent=None):
    message = QtGui.QMessageBox(title, text, QtGui.QMessageBox.Information,
                                QtGui.QMessageBox.Ok, QtGui.QMessageBox.NoButton, QtGui.QMessageBox.NoButton,
                                parent)
    message.setStyleSheet('background: solid;')
    message.exec_()


def MSG_WARNING(text, title='Warning', parent=None):
    message = QtGui.QMessageBox(title, text, QtGui.QMessageBox.Warning,
                                QtGui.QMessageBox.Ok, QtGui.QMessageBox.NoButton, QtGui.QMessageBox.NoButton,
                                parent)
    message.setStyleSheet('background: solid;')
    message.exec_()


def MSG_ERROR(text, title='Error', parent=None):
    message = QtGui.QMessageBox(title, text, QtGui.QMessageBox.Critical,
                                QtGui.QMessageBox.Ok, QtGui.QMessageBox.NoButton, QtGui.QMessageBox.NoButton,
                                parent)
    message.setStyleSheet('background: solid;')
    message.exec_()


def QUESTION_YES_NO(title, text, parent=None):
    question = QtGui.QMessageBox(title, text, QtGui.QMessageBox.Question,
                                QtGui.QMessageBox.Yes, QtGui.QMessageBox.No, QtGui.QMessageBox.NoButton,
                                parent)
    question.setDefaultButton(QtGui.QMessageBox.Yes)
    question.setStyleSheet('background: solid;')
    if question.exec_() == QtGui.QMessageBox.Yes:
        return True
    return False


def DIALOG_DIRECTORY_OPEN(title='Open directory', directory='', parent=None):
    directory = QtGui.QFileDialog.getExistingDirectory(parent, title, directory)
    return directory if directory else None


def DIALOG_FILE_OPEN(title='Open file', directory='', filter='', parent=None):
    filename = QtGui.QFileDialog.getOpenFileName(parent, title, directory, filter)
    return filename if filename else None


def DIALOG_FILE_SAVE(title='Open file', directory='', filter='', parent=None):
    filename = QtGui.QFileDialog.getSaveFileName(parent, title, directory, filter)
    return filename if filename else None


def DIALOG_SELECT_FONT(initial=None, parent=None, title='Select Font'):
    dialog = QtGui.QFontDialog(parent)
    dialog.setWindowTitle(title)
    dialog.setStyleSheet('background: solid;')
    if initial is not None:
        dialog.setCurrentFont(initial)
    if dialog.exec_() == QtGui.QDialog.Accepted:
        return dialog.selectedFont()
    return None


def DIALOG_SELECT_COLOR(initial=Qt.white, parent=None, title='Select Color'):
    dialog = QtGui.QColorDialog(initial, parent)
    dialog.setWindowTitle(title)
    dialog.setStyleSheet('background: solid;')
    if dialog.exec_() == QtGui.QDialog.Accepted:
        return dialog.selectedColor()
    return None 
