__author__    = 'Viktor Kerkez <alefnula@teacupapps.com>'
__contact__   = 'alefnula@teacupapps.com'
__date__      = '02 January 2010'
__copyright__ = 'Copyright (c) 2010 Viktor Kerkez'

import os
from dialogs import * #@UnusedWildImport
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt


class Editor(QtGui.QTextEdit):
    def __init__(self, parent, path=None):
        super(Editor, self).__init__(parent)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setFrameShape(QtGui.QFrame.NoFrame)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setAutoFormatting(QtGui.QTextEdit.AutoAll)
        # Setup Data
        self.filename = None

    # Property section
    def get_text_color(self):
        return self.palette().color(QtGui.QPalette.Text)
    def set_text_color(self, color):
        palette = self.palette()
        palette.setColor(QtGui.QPalette.Text, color)
        self.setPalette(palette)
    text_color = property(get_text_color, set_text_color)

    def get_text_font(self):
        return self.font()
    def set_text_font(self, font):
        self.setFont(font)
    text_font = property(get_text_font, set_text_font)

    def save_if_changed(self):
        if self.document().isModified() and QUESTION_YES_NO('Save file?', 'Document%s is modified,%sdo you want to save it?' % (
                                                            (' "%s"' % self.filename, '\n') if self.filename else ('', ' ')),
                                                            self.parent()):
            return self.save()
        return True
    
    def new(self):
        if self.save_if_changed():
            self.setDocument(QtGui.QTextDocument(self))

    def open(self):
        if self.save_if_changed():
            filename = DIALOG_FILE_OPEN('Open document', os.getcwd(), 'Text Files (*.txt)', self.parent())
            if filename is not None:
                fh = None
                try:
                    fh = QtCore.QFile(filename)
                    if not fh.open(QtCore.QIODevice.ReadOnly):
                        raise IOError, unicode(fh.errorString())
                    stream = QtCore.QTextStream(fh)
                    stream.setCodec('UTF-8')
                    self.setPlainText(stream.readAll())
                    self.document().setModified(False)
                    self.filename = filename
                    return True
                except (IOError, OSError), e:
                    QtGui.QMessageBox.warning(self, 'Load Error', 'Failed to load %s: %s' % (filename, e))
                    return False
                finally:
                    if fh is not None:
                        fh.close()

    def save(self):
        return self.save_as(self.filename)

    def save_as(self, filename=None):
        if filename is None:
            filename = DIALOG_FILE_SAVE('Save Document As', os.getcwd(), 'Text Files (*.txt)', self.parent())
            if filename is None:
                MSG_ERROR('Document could not be saved', 'Save Error', self.parent())
                return False
        fh = None
        try:
            fh = QtCore.QFile(filename)
            if not fh.open(QtCore.QIODevice.WriteOnly):
                raise IOError(unicode(fh.errorString()))
            stream = QtCore.QTextStream(fh)
            stream.setCodec('UTF-8')
            stream << self.toPlainText()
            self.document().setModified(False)
            self.filename = filename
            return True
        except (IOError, OSError):
            MSG_ERROR('Failed to save "%s"' % filename, 'Save Error', self.parent())
            return False
        finally:
            if fh is not None:
                fh.close()
    
    def change_text_font(self):
        font = DIALOG_SELECT_FONT(self.text_font, self.parent(), 'Select Text Font')
        if font is not None:
            self.text_font = font

    def change_text_color(self):
        color = DIALOG_SELECT_COLOR(self.text_color, self.parent(), 'Select Text Color')
        if color is not None:
            self.text_color = color
