__author__    = 'Viktor Kerkez <alefnula@teacupapps.com>'
__contact__   = 'alefnula@teacupapps.com'
__date__      = '03 January 2010'
__copyright__ = 'Copyright (c) 2010 Viktor Kerkez'

from Queue import Queue
# PyQt4 imports
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt
# ZenWriter imports
import utils
import definitions


class Fader(QtCore.QThread):
    def __init__(self, objects, min=0.0, max=1.0, timeout=300, range=5):
        QtCore.QObject.__init__(self)
        self._objects  = objects
        self._min      = min
        self._max      = max
        self._timeout  = timeout
        self._range    = range
        self._q        = (max - min) / range
        self.timeline = QtCore.QTimeLine(timeout)
        self.timeline.setFrameRange(1, range)
        self.connect(self, QtCore.SIGNAL('startTimeline'), self.timeline.start)
        self.connect(self, QtCore.SIGNAL('stopTimeline'), self.timeline.stop)
        self.connect(self.timeline, QtCore.SIGNAL('frameChanged(int)'), self._do)
        self._out = False
        self._queue = Queue()
        self.start()

    def _do(self, frame):
        for object in self._objects:
            object.setOpacity(self._max - (frame * self._q) if self._out else self._min + (self._q * frame))

    def fade_in(self):
        self._queue.put(False)
    
    def fade_out(self):
        self._queue.put(True)

    def run(self):
        while True:
            self._out = self._queue.get()
            while not self._queue.empty():
                self._out = self._queue.get()
            self.emit(QtCore.SIGNAL('stopTimeline'))
            self.emit(QtCore.SIGNAL('startTimeline'))


class ZendoButton(QtGui.QGraphicsPixmapItem):
    def __init__(self, parent, action):
        super(ZendoButton, self).__init__(QtGui.QPixmap(action['image']))
        self.setToolTip(action['tooltip'])
        self.setPos(action['x'], action['y'])
        self._callback = utils.get_object(parent, action['callback'])
        self.setAcceptsHoverEvents(True)
        self.setAcceptedMouseButtons(Qt.LeftButton)
        self.setOpacity(0.0)
        # Setup Action
        self.action = QtGui.QAction(QtGui.QIcon(action['image']), action['tooltip'], parent)
        self.action.setShortcut(action['shortcut'])
        self.action.setCheckable(False)
        self.action.connect(self.action, QtCore.SIGNAL('triggered()'), self._callback)
        parent.addAction(self.action)

    def set_callback(self, callback):
        self._callback = callback

    def mousePressEvent(self, event):
        if self._callback is not None:
            self._callback()

    def mouseReleaseEvent(self, event):
        pass



class Sidebar(QtGui.QGraphicsView):
    def __init__(self, parent):
        self.scene = QtGui.QGraphicsScene()
        QtGui.QGraphicsView.__init__(self, self.scene, parent)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setFrameShape(QtGui.QFrame.NoFrame)
        self.setFocusPolicy(Qt.NoFocus)
        # Create Buttons
        for action in definitions.ACTIONS:
            button = ZendoButton(parent, action)
            self.scene.addItem(button)
        self.fader = Fader(self.scene.items(), 0.0, 0.5)

    def enterEvent(self, event):
        self.fader.fade_in()

    def leaveEvent(self, event):
        self.fader.fade_out()

    def resizeEvent(self, resize_event):
        self.scene.setSceneRect(QtCore.QRectF(0.0, 0.0, self.width(), self.height()))
