__author__    = 'Viktor Kerkez <alefnula@teacupapps.com>'
__contact__   = 'alefnula@teacupapps.com'
__date__      = '02 January 2010'
__copyright__ = 'Copyright (c) 2010 Viktor Kerkez'


import os
import sys
import glob
# PyQt4 imports
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt
from PyQt4.phonon import Phonon
# ZenWriter import
import editor
import sidebar
import resources #@UnusedImport
from dialogs import * #@UnusedWildImport


class ZendoPad(QtGui.QFrame):
    def __init__(self, args):
        super(ZendoPad, self).__init__(None, Qt.FramelessWindowHint)
        self.setWindowState(Qt.WindowFullScreen)
        # Layout
        self.layout = QtGui.QHBoxLayout()
        self.layout.setMargin(0)
        self.layout.setSpacing(0)
        # Editor
        self.editor = editor.Editor(self)
        self.layout.addWidget(self.editor)
        # Sidebar
        self.sidebar = sidebar.Sidebar(self)
        self.layout.addWidget(self.sidebar)
        # Set Layout
        self.setLayout(self.layout)
        # Load Application State
        self.load_application_state()
        # Initialize Multimedia
        self.media = Phonon.MediaObject(self)
        self.connect(self.media, QtCore.SIGNAL('finished()'), self.music_next)
        Phonon.createPath(self.media, Phonon.AudioOutput(Phonon.MusicCategory, self))
        self.media_queue = []
        self.media_position = -1
        # Set focus
        self.editor.setFocus()

    def resizeEvent(self, resize_event):
        super(ZendoPad, self).resizeEvent(resize_event)
        self.editor.setMaximumWidth(self.width()-200)
        self.editor.setMinimumWidth(self.width()-200)

    def get_background(self):
        return self._background
    def set_background(self, background):
        self._background = background
        self.setStyleSheet('background-image: url(%s); background-position: center; background-repeat: none;' % self._background)
    background = property(get_background, set_background)

    def change_background(self):
        background = DIALOG_FILE_OPEN('Select Background Image', os.path.dirname(str(self.background)), 'Images (*.png *.jpg *.jpeg)', self)
        if background is not None:
            self.background = background

    def music_add(self):
        directory = DIALOG_DIRECTORY_OPEN('Open Music Directory', self.media_directory, self)
        if directory is not None:
            self.media_directory = directory
            if self.media.state() == Phonon.PlayingState:
                self.media.stop()
            files = []
            for pattern in ('*.mp3', '*.wav', '*.wma', '*.ogg'):
                files += glob.glob(os.path.join(str(directory), pattern))
            self.media_queue = map(lambda file: Phonon.MediaSource(QtCore.QUrl.fromLocalFile('file://%s' % file)), files)
            self.media_position = -1
            self.music_next()

    def music_previous(self):
        if self.media_position > 0 and len(self.media_queue) > (self.media_position - 1):
            self.media.stop()
            self.media_position -= 1
            self.media.setCurrentSource(self.media_queue[self.media_position])
            self.media.play()
        
    def music_play(self):
        if self.media.state() != Phonon.PlayingState:
            if self.media.state() == Phonon.StoppedState:
                if self.media_position != -1 and len(self.media_queue) > self.media_position:
                    self.media.setCurrentSource(self.media_queue[self.media_position])
            self.media.play()

    def music_pause(self):
        if self.media.state() == Phonon.PlayingState:
            self.media.pause()
    
    def music_stop(self):
        if self.media.state() == Phonon.PlayingState:
            self.media.stop()
            self.media.setCurrentSource(self.media_queue[self.media_position])
    
    def music_next(self):
        if len(self.media_queue) > (self.media_position + 1):
            self.media.stop()
            self.media_position += 1
            self.media.setCurrentSource(self.media_queue[self.media_position])
            self.media.play()

    def closeEvent(self, event):
        self.editor.save_if_changed()
        # Save Application State before closing
        self.save_application_state()
        super(ZendoPad, self).closeEvent(event)

    def save_application_state(self):
        settings = QtCore.QSettings()
        settings.setValue('Background', QtCore.QVariant(self.background))
        settings.setValue('TextColor', QtCore.QVariant(self.editor.text_color))
        settings.setValue('TextFont', QtCore.QVariant(self.editor.text_font))
        settings.setValue('MediaDirectory', QtCore.QVariant(self.media_directory))

    def load_application_state(self):
        settings = QtCore.QSettings()
        self.background = settings.value('Background', QtCore.QVariant(':/background.jpg')).toString()
        self.editor.text_color = QtGui.QColor(settings.value('TextColor'))
        self.editor.text_font = QtGui.QFont(settings.value('TextFont'))
        self.media_directory = settings.value('MediaDirectory', QtCore.QVariant(os.getcwd())).toString()

# Main Function
def main(args):
    app = QtGui.QApplication(args)
    app.setOrganizationName('TeacupApps')
    app.setApplicationName('ZendoPad')
    app.setWindowIcon(QtGui.QIcon(':/ZendoPad.ico'))
    zenwriter = ZendoPad(args)
    zenwriter.show()
    return app.exec_()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))
